# Copyright 2021 (c) Anna Schumaker.
#
# Table: genres
# +---------+-----------+------+------+
# | genreid | plstateid | name | sort |
# +---------+-----------+------+------+
#
# Index: genre_index
# +-----------------+
# | name -> genreid |
# +-----------------+
#
# Map: genre_map
# +---------+---------+
# | genreid | trackid |
# +---------+---------+
from gi.repository import GObject
from . import objects
from . import playlist
from . import track
from . import sql

class Genre(playlist.Playlist):
    def __init__(self, row):
        playlist.Playlist.__init__(self, row, "emblem-generic")
        self._name = row["name"]

    def delete(self): Table.delete(self)

    @GObject.Property
    def name(self): return self._name


class GenreTable(playlist.Model):
    def __init__(self):
        playlist.Model.__init__(self, "genres", "sort")

    def do_create(self):
        sql.execute("CREATE TABLE IF NOT EXISTS genres "
                       "(genreid INTEGER PRIMARY KEY, "
                       " plstateid INTEGER NOT NULL, "
                       " name TEXT UNIQUE, "
                       " sort TEXT, "
                       " FOREIGN KEY(plstateid) REFERENCES playlist_states(plstateid))")
        sql.execute("CREATE INDEX IF NOT EXISTS genre_index "
                    "ON genres(name)")

    def do_factory(self, row):
        return Genre(row)

    def do_insert(self, plstate, name):
        return sql.execute("INSERT INTO genres (plstateid, name, sort) "
                           "VALUES (?, ?, ?)",
                          [ plstate.rowid, name, name.casefold() ])

    def do_lookup(self, name):
        return sql.execute("SELECT * FROM genres WHERE name=?", [ name ])


class GenreMap(objects.Map):
    def __init__(self):
        objects.Map.__init__(self, "genre_map", Table.get, track.Table.get)
        self.lookup_tracks = self.lookup_rhs
        self.lookup_genres = self.lookup_lhs

    def do_create(self):
        sql.execute("CREATE TABLE IF NOT EXISTS genre_map "
                        "(genreid INTEGER, "
                        " trackid INTEGER, "
                        " FOREIGN KEY(genreid) REFERENCES genres(genreid), "
                        " FOREIGN KEY(trackid) REFERENCES tracks(trackid), "
                        " UNIQUE(genreid, trackid))")

    def do_insert(self, genre, track):
        sql.execute("INSERT INTO genre_map (genreid, trackid) "
                    "VALUES (?, ?)", [ genre.rowid, track.rowid ])

    def do_delete(self, genre, track):
        return sql.execute("DELETE FROM genre_map WHERE genreid=? AND trackid=?",
                           [ genre.rowid, track.rowid ])

    def do_lookup_rhs(self, genre):
        return sql.execute("SELECT trackid FROM genre_map "
                           "WHERE genreid=?", [ genre.rowid ])

    def do_lookup_lhs(self, track):
        return sql.execute("SELECT genreid FROM genre_map "
                           "WHERE trackid=?", [ track.rowid ])

    def delete_track(self, track):
        for genre in self.lookup_genres(track):
            self.delete(genre, track)


Table = GenreTable()
Map   = GenreMap()
