# Copyright 2021 (c) Anna Schumaker.
import db
import sqlite3
import unittest
from gi.repository import GObject


class TestGenre(unittest.TestCase):
    def setUp(self):
        db.reset()

    def test_init(self):
        genre = db.genre.Table.find("Test Genre")
        self.assertIsInstance(genre, db.playlist.Playlist)
        self.assertEqual(genre._name, "Test Genre")
        self.assertEqual(genre.get_property("name"), "Test Genre")
        self.assertEqual(genre.get_property("icon-name"), "emblem-generic")

    def test_delete(self):
        genre = db.genre.Table.find("Test Genre")
        genre.delete()
        self.assertIsNone(db.genre.Table.lookup("Test Genre"))


class TestGenreTable(unittest.TestCase):
    def setUp(self):
        db.reset()

    def test_init(self):
        table = db.genre.GenreTable()
        self.assertIsInstance(table, db.playlist.Model)
        self.assertEqual(table.table, "genres")
        self.assertEqual(table.order, "sort")

        self.assertIsInstance(db.genre.Table, db.genre.GenreTable)
        db.sql.execute("SELECT genreid,plstateid,name,sort FROM genres")

    def test_insert(self):
        table = db.genre.GenreTable()
        genre = table.insert("Test Genre")

        self.assertIsInstance(genre, db.genre.Genre)
        self.assertEqual(genre._name, "Test Genre")

        with self.assertRaises(sqlite3.IntegrityError):
            db.genre.Table.insert("Test Genre")

    def test_lookup(self):
        genre = db.genre.Table.insert("Test Genre")
        self.assertEqual(db.genre.Table.lookup("Test Genre"), genre)
        self.assertIsNone(db.genre.Table.lookup("none"))


class TestGenreMap(unittest.TestCase):
    def setUp(self):
        db.reset()

    def test_genre_map_init(self):
        self.assertIsInstance(db.genre.Map, db.genre.GenreMap)
        self.assertEqual(db.genre.Map.map_lhs, db.genre.Table.get)
        self.assertEqual(db.genre.Map.map_rhs, db.track.Table.get)
        db.sql.execute("SELECT genreid,trackid FROM genre_map")

    def test_genre_map_insert(self):
        track = db.make_fake_track(1, 1.234, "Test Title", "/a/b.cde")
        genre = db.genre.Table.find("Test Genre")

        db.genre.Map.insert(genre, track)
        with self.assertRaises(sqlite3.IntegrityError):
            db.genre.Map.insert(genre, track)

    def test_genre_map_delete(self):
        track = db.make_fake_track(1, 1.234, "Test Title", "/a/b.cde")
        genre = db.genre.Table.find("Test Genre")

        db.genre.Map.insert(genre, track)
        db.genre.Map.delete(genre, track)
        self.assertEqual(db.genre.Map.lookup_tracks(genre), [ ])

    def test_genre_map_lookup_tracks(self):
        track1 = db.make_fake_track(1, 1.234, "Test Title", "/a/b.cde")
        track2 = db.make_fake_track(2, 2.345, "Test Title 2", "/a/c.def")
        genre  = db.genre.Table.find("Test Genre")

        db.genre.Map.insert(genre, track1)
        db.genre.Map.insert(genre, track2)

        lookup_res = db.genre.Map.lookup_tracks(genre)
        self.assertEqual(lookup_res, [ track1, track2 ])

    def test_genre_map_lookup_genres(self):
        track  = db.make_fake_track(1, 1.234, "Test Title", "/a/b.cde")
        genre1 = db.genre.Table.find("Test Genre 1")
        genre2 = db.genre.Table.find("Test Genre 2")

        db.genre.Map.insert(genre1, track)
        db.genre.Map.insert(genre2, track)

        lookup_res = db.genre.Map.lookup_genres(track)
        self.assertEqual(lookup_res, [ genre1, genre2 ])

    def test_genre_map_delete_track(self):
        track  = db.make_fake_track(1, 1.234, "Test Title", "/a/b.cde")
        genre1 = db.genre.Table.find("Test Genre 1")
        genre2 = db.genre.Table.find("Test Genre 2")

        db.genre.Map.insert(genre1, track)
        db.genre.Map.insert(genre2, track)

        db.genre.Map.delete_track(track)
        self.assertEqual(db.genre.Map.lookup_genres(track), [ ])
